﻿using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.FitnessLandscape.Analysis {
  [Item("InformationAnalysisCalculator", "Performs an information analysis on a quality trail as described in Vassilev et al. (2000) Evol Comput 8:31–60")]
  [StorableClass]
  public class InformationAnalysisCalculator : SingleSuccessorOperator {

    #region Parameter Properties
    public LookupParameter<DataTable> QualityTrailParameter {
      get { return (LookupParameter<DataTable>)Parameters["QualityTrail"]; }
    }
    public ValueLookupParameter<IntValue> NQuantilesParameter {
      get { return (ValueLookupParameter<IntValue>)Parameters["NQuantiles"]; }
    }
    public ValueLookupParameter<IntValue> ShapeSizeParameter {
      get { return (ValueLookupParameter<IntValue>) Parameters["ShapeSize"]; }
    }
    public LookupParameter<DoubleValue> InformationStabilityParameter {
      get { return (LookupParameter<DoubleValue>)Parameters["InformationStability"]; }
    }
    public LookupParameter<IntValue> RegularityParameter {
      get { return (LookupParameter<IntValue>)Parameters["Regularity"]; }
    }
    public LookupParameter<IntValue> DiversityParameter {
      get { return (LookupParameter<IntValue>)Parameters["Diversity"]; }
    }
    public LookupParameter<DoubleArray> InformationContentParameter {
      get { return (LookupParameter<DoubleArray>)Parameters["InformationContent"]; }
    }
    public LookupParameter<DoubleArray> PartialInformationContentParameter {
      get { return (LookupParameter<DoubleArray>)Parameters["PartialInformationContent"]; }
    }
    public LookupParameter<DoubleArray> DensityBasinInformationParameter {
      get { return (LookupParameter<DoubleArray>)Parameters["DensityBasinInformation"]; }
    }
    public LookupParameter<DoubleArray> TotalEntropyParameter {
      get { return (LookupParameter<DoubleArray>)Parameters["TotalEntropy"]; }
    }
    public LookupParameter<DoubleArray> EpsilonQuantilesParameter {
      get { return (LookupParameter<DoubleArray>)Parameters["EpsilonQuantiles"]; }
    }
    #endregion

    #region Constructors & Cloning
    [StorableConstructor]
    protected InformationAnalysisCalculator(bool deserializing) : base(deserializing) { }
    public InformationAnalysisCalculator() {
      Parameters.Add(new LookupParameter<DataTable>("QualityTrail", "Historical quality trail of a walk over a fitness landscape."));

      Parameters.Add(new ValueLookupParameter<IntValue>("NQuantiles", "Number of epsilon quantiles to calculate information analysis."));
      Parameters.Add(new ValueLookupParameter<IntValue>("ShapeSize", "The number of consecutive slopes to consider as shape.", new IntValue(2)));

      Parameters.Add(new LookupParameter<DoubleValue>("InformationStability"));
      Parameters.Add(new LookupParameter<IntValue>("Regularity"));
      Parameters.Add(new LookupParameter<IntValue>("Diversity"));
      Parameters.Add(new LookupParameter<DoubleArray>("InformationContent"));
      Parameters.Add(new LookupParameter<DoubleArray>("PartialInformationContent"));
      Parameters.Add(new LookupParameter<DoubleArray>("DensityBasinInformation"));
      Parameters.Add(new LookupParameter<DoubleArray>("TotalEntropy"));
      Parameters.Add(new LookupParameter<DoubleArray>("EpsilonQuantiles", "Considered values of epsilon, selected as quantiles of actual quality differences"));
    }
    protected InformationAnalysisCalculator(InformationAnalysisCalculator original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new InformationAnalysisCalculator(this, cloner);
    }
    #endregion

    public override IOperation Apply() {
      var analysis = new InformationAnalysis(
        QualityTrailParameter.ActualValue.Rows.First().Values,
        NQuantilesParameter.ActualValue.Value,
        ShapeSizeParameter.ActualValue.Value);
      RegularityParameter.ActualValue = new IntValue(analysis.Regularity);
      DiversityParameter.ActualValue = new IntValue(analysis.Diversity);
      InformationStabilityParameter.ActualValue = new DoubleValue(analysis.InformationStability);
      InformationContentParameter.ActualValue = new DoubleArray(analysis.InformationContent.ToArray());
      PartialInformationContentParameter.ActualValue = new DoubleArray(analysis.PartialInformationContent.ToArray());
      DensityBasinInformationParameter.ActualValue = new DoubleArray(analysis.DensityBasinInformation.ToArray());
      TotalEntropyParameter.ActualValue = new DoubleArray(analysis.TotalEntropy.ToArray());
      EpsilonQuantilesParameter.ActualValue = new DoubleArray(analysis.QualityDelta.ToArray());
      return base.Apply();
    }
  }
}
